// Copyright 2013 Atbrox AS
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#import <UIKit/UIKit.h>

#import "PostingListCompression.h"

@class FirstViewController;

@protocol StatisticsUpdateDelegate <NSObject>

- (void)setEncodeStatistics:(NSDictionary *)statistics forScheme:(NSString *)scheme iterations:(NSUInteger)iterations blockSize:(NSUInteger)blockSize;
- (void)setDecodeStatistics:(NSDictionary *)statistics forScheme:(NSString *)scheme iterations:(NSUInteger)iterations blockSize:(NSUInteger)blockSize;

@end

@interface FirstViewController : UIViewController <UITextFieldDelegate, PostingListCompressionDelegate>

@property (weak, nonatomic) id<StatisticsUpdateDelegate> delegate;

@property (strong, nonatomic) IBOutlet UITapGestureRecognizer *gestureRecognizer;

@property (strong, nonatomic) IBOutlet UIButton *encodeButton;
@property (strong, nonatomic) IBOutlet UIProgressView *encodeProgress;

@property (strong, nonatomic) IBOutlet UIButton *decodeButton;
@property (strong, nonatomic) IBOutlet UIProgressView *decodeProgress;

@property (strong, nonatomic) IBOutlet UILabel *noTermsLabel;
@property (strong, nonatomic) IBOutlet UITextField *noTermsTextField;
@property (strong, nonatomic) IBOutlet UISlider *noTermsSlider;

@property (strong, nonatomic) IBOutlet UITextField *iterationsTextField;
@property (strong, nonatomic) IBOutlet UITextField *threadsTextField;
@property (strong, nonatomic) IBOutlet UITextField *blockSizeTextField;

@property (strong, nonatomic) IBOutlet UISegmentedControl *compressionSchemeSegmentedController;



-(IBAction)noTermsSliderValueChanged:(id)sender;
-(IBAction)encodeButtonTouchUpInside:(id)sender;
-(IBAction)decodeButtonTouchUpInside:(id)sender;

- (IBAction)didTouchUpInsideCompressionSchemeSegmentedController:(id)sender;

@end
