// Copyright 2013 Atbrox AS
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#import "FirstViewController.h"

#import <sys/mount.h>

#import "PostingListCompression.h"
#import "VariableByteDecoder.h"
#import "VariableByteEncoder.h"
#import "GroupVarintDecoder.h"
#import "GroupVarintEncoder.h"
#import "EliasGammaEncoder.h"
#import "EliasGammaDecoder.h"
#import "CompressionToolbox.h"


@interface FirstViewController ()

@end

@implementation FirstViewController
{
    PostingListCompression *_postingList;
    NSString *_schemeName;
    
    NSFileManager *_fileManager;

    NSString *_encodeInput;
    NSString *_encodeInputMapping;
    NSMutableArray *_encodeIterations;
    
    NSMutableArray *_decodeIterations;
    
    NSUInteger _noTerms;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
//    NSData *data = [EliasGammaEncoder encode:@[@24, @35, @43, @51, @58, @59, @63]];
    
    _fileManager = [NSFileManager defaultManager];
    
    _encodeInput = [[NSBundle mainBundle] pathForResource:@"postinglist" ofType:@"out"];
    _encodeInputMapping = [[NSBundle mainBundle] pathForResource:@"mapping-postinglist" ofType:@"out"];
    
    [_noTermsSlider setMinimumValue:1];
    [_noTermsSlider setMaximumValue:[PostingListCompression numberOfTerms:_encodeInputMapping withFixedByteSize:8]];
    NSUInteger value = roundf([_noTermsSlider maximumValue] / 2);
    [_noTermsSlider setValue:value];
    [_noTermsTextField setText:[NSString stringWithFormat:@"%i", value]];
    _noTerms = value;
    
    [_encodeProgress setProgress:0];
    [_decodeProgress setProgress:0];

//    NSString *decodedPath = @"/Users/lars/Workspace/PostingList/PostingListApp/Resources/postinglist.out";
//
//    NSFileHandle *decodedFH = [NSFileHandle fileHandleForReadingAtPath:decodedPath];
//    [decodedFH seekToFileOffset:9];
//    NSData *decodedData = [decodedFH readDataToEndOfFile];
//    NSLog(@"Total unencoded size: %i", decodedData.length);
//    [decodedFH closeFile];
//    NSString *decodedString = [[NSString alloc] initWithData:decodedData encoding:NSUTF8StringEncoding];
//    
//    CFMutableArrayRef decodedArray = CFArrayCreateMutable(kCFAllocatorDefault, 0, NULL);
//    char *postingListToken = strtok((char *)[decodedString UTF8String], ",");
//    while (postingListToken != NULL) {
//        CFArrayAppendValue(decodedArray, (void *)strtol(postingListToken, NULL, 0));
//        postingListToken = strtok(NULL, ",");
//    }
//    
//    NSUInteger totalNumberOfPostings = CFArrayGetCount(decodedArray);
//    NSLog(@"Number of postings: %i", totalNumberOfPostings);
//    
////    for (NSUInteger j = totalNumberOfPostings - 1; j > 0; j--)
////    {
////        NSUInteger larger = (NSUInteger)CFArrayGetValueAtIndex(decodedArray, j);
////        NSUInteger lesser = (NSUInteger)CFArrayGetValueAtIndex(decodedArray, j - 1);
////        NSUInteger delta = larger - lesser;
////        //                            NSLog(@"Calculated delta: %li - %li = %@ inserted at %li", larger, lesser, delta, j);
////        CFArraySetValueAtIndex(decodedArray, j, (void *)delta);
////    }
//    
//    NSDate *vbyteStartEncode = [NSDate date];
//    NSData *vbyteData = [[[VariableByteEncoder alloc] init] encode:decodedArray];
//    NSDate *vbyteEndEncode = [NSDate date];
//    float vbyteTotalEncodeTime = [vbyteEndEncode timeIntervalSinceDate:vbyteStartEncode];
//    NSLog(@"[VBYTE ENCODE] Took: %f seconds. Average: %f seconds/posting. Size: %i", vbyteTotalEncodeTime, vbyteTotalEncodeTime / totalNumberOfPostings, vbyteData.length);
//    
//    NSDate *vbyteStartDecode = [NSDate date];
//    [[[VariableByteDecoder alloc] init] decode:vbyteData];
//    NSDate *vbyteEndDecode = [NSDate date];
//    float vbyteTotalDecodeTime = [vbyteEndDecode timeIntervalSinceDate:vbyteStartDecode];
//    NSLog(@"[VBYTE DECODE] Took: %f seconds. Average: %f seconds/posting.", vbyteTotalDecodeTime, vbyteTotalDecodeTime / totalNumberOfPostings);
//    
//
//    NSDate *gviStartEncode = [NSDate date];
//    NSData *gviData = [[[GroupVarintEncoder alloc] init] encode:decodedArray];
//    NSDate *gviEndEncode = [NSDate date];
//    float gviTotalTime = [gviEndEncode timeIntervalSinceDate:gviStartEncode];
//    NSLog(@"[GVI ENCODE] Took: %f seconds. Average: %f seconds/posting. Size: %i", gviTotalTime, gviTotalTime / totalNumberOfPostings, gviData.length);
//    
//    NSDate *gviStartDecode = [NSDate date];
//    [[[GroupVarintDecoder alloc] init] decode:gviData];
//    NSDate *gviEndDecode = [NSDate date];
//    float gviTotalDecodeTime = [gviEndDecode timeIntervalSinceDate:gviStartDecode];
//    NSLog(@"[GVI DECODE] Took: %f seconds. Average: %f seconds/posting.", gviTotalDecodeTime, gviTotalDecodeTime / totalNumberOfPostings);
//    
//    
//    NSDate *eliasStartEncode = [NSDate date];
//    NSData *eliasData = [[[EliasGammaEncoder alloc] init] encode:decodedArray];
//    NSDate *eliasEndEncode = [NSDate date];
//    float eliasTotalEncodeTime = [eliasEndEncode timeIntervalSinceDate:eliasStartEncode];
//    NSData *eliasBitData = [CompressionToolbox convertToBits:eliasData overflow:YES];
//    NSLog(@"[ELIAS ENCODE] Took: %f seconds. Average: %f seconds/posting. Size: %i", eliasTotalEncodeTime, eliasTotalEncodeTime / totalNumberOfPostings, eliasBitData.length);
//    
//    NSDate *eliasStartDecode = [NSDate date];
//    [[[EliasGammaDecoder alloc] init] decode:eliasData];
//    NSDate *eliasEndDecode = [NSDate date];
//    float eliasTotalDecodeTime = [eliasEndDecode timeIntervalSinceDate:eliasStartDecode];
//    NSLog(@"[ELIAS DECODE] Took: %f seconds. Average: %f seconds/posting.", eliasTotalDecodeTime, eliasTotalDecodeTime / totalNumberOfPostings);
    
    _postingList = [[PostingListCompression alloc] init];
    _postingList.delegate = self;
    _postingList.encodingModel.delegate = self;
    _postingList.decodingModel.delegate = self;
    [_postingList setPathToUnencodedData:_encodeInput];
    
    _delegate = self.tabBarController.viewControllers[1];
    
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

-(IBAction) dismissKeyboard:(id)sender
{
    [_noTermsTextField resignFirstResponder];
    [_iterationsTextField resignFirstResponder];
    [_blockSizeTextField resignFirstResponder];
    [_threadsTextField resignFirstResponder];
}

-(IBAction) noTermsSliderValueChanged:(id)sender
{
    NSUInteger newValue = roundf([_noTermsSlider value]);
    [_noTermsSlider setValue:newValue];
    [_noTermsTextField setText:[NSString stringWithFormat:@"%i", newValue]];
    _noTerms = newValue;
}

-(IBAction)encodeButtonTouchUpInside:(id)sender {
    NSLog(@"Starting encode. Processing %i terms...", _noTerms);
    
    id <EncoderProtocol> encodeScheme;
    NSUInteger schemeIndex = _compressionSchemeSegmentedController.selectedSegmentIndex;
    NSLog(@"Encoding using: '%@'...", [_compressionSchemeSegmentedController titleForSegmentAtIndex:schemeIndex]);
    if (schemeIndex == 0) {
        encodeScheme = [[VariableByteEncoder alloc] init];
    }
    else if (schemeIndex == 1) {
        encodeScheme = [[GroupVarintEncoder alloc] init];
    }
    else {
        encodeScheme = [[EliasGammaEncoder alloc] init];
    }
    _schemeName = [encodeScheme scheme];
    
    for (NSUInteger i = 1; i <= _noTerms; i++)
    {
        NSString *term = [NSString stringWithFormat:@"%08i", i];
        NSRange range = [PostingListCompression postingListRange:_encodeInputMapping forTerm:term atLineNo:i withFixedByteSize:10];
        [_postingList addTermForEncode:term inRange:range];
    }
    NSLog(@"%i terms enqueued for processing...", _noTerms);
    dispatch_queue_t encodeQueue = dispatch_queue_create("EncodeQueue", NULL);
    _postingList.iterations = [_iterationsTextField.text integerValue];
    _postingList.blockSize = [_blockSizeTextField.text integerValue];
    _postingList.encodingModel.numberOfThreads = [_threadsTextField.text integerValue];
    _encodeIterations = [NSMutableArray array];
    for (NSUInteger i = 0; i < _postingList.iterations; i++)
        [_encodeIterations addObject:[NSDictionary dictionary]];
    
    dispatch_async(encodeQueue, ^{
        [_postingList encodeUsing:encodeScheme];
//        [postingList storeEncodedData:decodeInput withMappingAtPath:decodeInputMapping inBlocks:YES];
    });
}

-(IBAction)decodeButtonTouchUpInside:(id)sender {
    
    NSString *decodeInputMapping;
    NSString *decodeInput;
    
    id <DecoderProtocol> decodeScheme;
    NSUInteger schemeIndex = _compressionSchemeSegmentedController.selectedSegmentIndex;
    NSLog(@"Decoding using: '%@'...", [_compressionSchemeSegmentedController titleForSegmentAtIndex:schemeIndex]);
    
    // VByte
    
    if (schemeIndex == 0) {
        decodeScheme = [[VariableByteDecoder alloc] init];
        decodeInput = [[NSBundle mainBundle] pathForResource:@"vbyte-data" ofType:@"out"];
        decodeInputMapping = [[NSBundle mainBundle] pathForResource:@"vbyte-mapping" ofType:@"out"];
    }
    
    // Group Varint
    
    else if (schemeIndex == 1) {
        decodeScheme = [[GroupVarintDecoder alloc] init];
        decodeInput = [[NSBundle mainBundle] pathForResource:@"gvi-data" ofType:@"out"];
        decodeInputMapping = [[NSBundle mainBundle] pathForResource:@"gvi-mapping" ofType:@"out"];
    }
    
    // Elias Gamma
    
    else {
        decodeScheme = [[EliasGammaDecoder alloc] init];
        decodeInput = [[NSBundle mainBundle] pathForResource:@"elias-data" ofType:@"out"];
        decodeInputMapping = [[NSBundle mainBundle] pathForResource:@"elias-mapping" ofType:@"out"];
    }
    _schemeName = [decodeScheme scheme];
    
    [_postingList setPathToEncodedData:decodeInput];
    
    if ([_fileManager fileExistsAtPath:decodeInput] && [_fileManager fileExistsAtPath:decodeInputMapping])
    {
        NSLog(@"Starting decode. Processing %i terms...", _noTerms);
        for (NSUInteger i = 1; i <= _noTerms; i++)
        {
            NSString *term = [NSString stringWithFormat:@"%08i", i];
            NSRange range = [PostingListCompression postingListRange:decodeInputMapping forTerm:term atLineNo:i withFixedByteSize:10];
            [_postingList addTermForDecode:term inRange:range];
        }
        _postingList.iterations = [_iterationsTextField.text integerValue];
        _postingList.blockSize = [_blockSizeTextField.text integerValue];
        _postingList.decodingModel.numberOfThreads = [_threadsTextField.text integerValue];
        _decodeIterations = [NSMutableArray array];
        for (NSUInteger i = 0; i < _postingList.iterations; i++)
            [_decodeIterations addObject:[NSDictionary dictionary]];
        
        dispatch_queue_t decodeQueue = dispatch_queue_create("DecodeQueue", NULL);
        dispatch_async(decodeQueue, ^{
            [_postingList decodeUsing:decodeScheme];
//            [postingList storeDecodedData:decodeInput withMappingAtPath:decodeInputMapping inBlocks:YES];
        });
    }
    else
        NSLog(@"Cannot find required files: '%@' and '%@'!", decodeInput, decodeInputMapping);
}

- (IBAction)didTouchUpInsideCompressionSchemeSegmentedController:(id)sender {
    
}

-(BOOL) textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string
{
    NSString *newString = [[_noTermsTextField text] stringByAppendingString:string];
    // Handle backspace
    if (![string compare:@""])
        newString = [newString substringToIndex:[newString length] - 1];
    
    NSUInteger newValue = [newString integerValue];
    if (newValue < 1)
        newValue = 1;
    else if (newValue > [_noTermsSlider maximumValue])
        newValue = roundf([_noTermsSlider maximumValue]);

    [_noTermsSlider setValue:newValue animated:YES];
    _noTerms = newValue;
    
    return YES;
}

- (void)iterationFinished:(NSUInteger)iteration operation:(NSString *)operation values:(NSDictionary *)values statistics:(NSDictionary *)stats {
    NSLog(@"Iteration #%i finished!", iteration);
    if ([operation isEqualToString:@"encode"]) {
        [_encodeIterations setObject:stats atIndexedSubscript:iteration];
        if (iteration >= _postingList.iterations - 1) {
            [_delegate setEncodeStatistics:[self averageStatistics:_encodeIterations] forScheme:_schemeName iterations:_postingList.iterations blockSize:_postingList.blockSize];
        }
    }
    else if ([operation isEqualToString:@"decode"]) {
        [_decodeIterations setObject:stats atIndexedSubscript:iteration];
        if (iteration >= _postingList.iterations - 1) {
            [_delegate setDecodeStatistics:[self averageStatistics:_decodeIterations] forScheme:_schemeName iterations:_postingList.iterations blockSize:_postingList.blockSize];
        }
    }
    else
        NSLog(@"Unknown operation!");
}

- (void)readFinished:(PostingListModel *)model forPostingListEntry:(PostingListEntry *)entry {
    
}

- (void)preProcessingFinished:(PostingListModel *)model forPostingListEntry:(PostingListEntry *)entry {
    
}

- (void)processingFinished:(PostingListModel *)model forPostingListEntry:(PostingListEntry *)entry {
    
}

- (void)postProcessingFinished:(PostingListModel *)model forPostingListEntry:(PostingListEntry *)entry {

}

- (void)writeFinished:(PostingListModel *)model forPostingListEntry:(PostingListEntry *)entry {
    if ([model isKindOfClass:[ParallelEncodingModel class]]) {
        NSUInteger currentProgress = entry.lastReadPosition;
        NSUInteger totalProgress = entry.range.length;
        dispatch_async(dispatch_get_main_queue(), ^{
            _encodeProgress.progress = currentProgress / (float)totalProgress;
        });
    }
    else if ([model isKindOfClass:[ParallelDecodingModel class]]) {
        NSUInteger currentProgress = entry.lastReadPosition;
        NSUInteger totalProgress = entry.range.length;
        dispatch_async(dispatch_get_main_queue(), ^{
            _decodeProgress.progress = currentProgress / (float)totalProgress;
        });
    }
}

- (NSDictionary *)averageStatistics:(NSArray *)iterations {
    NSLog(@"Calculating statistics...");
    float averageIORead = 0.0;
    float averageIOReadWait = 0.0;
    float averagePreProcess = 0.0;
    float averageProcess = 0.0;
    float averagePostProcess = 0.0;
    float averageProcessWait = 0.0;
    float averageIOWrite = 0.0;
    float averageIOWriteWait = 0.0;
    float averageTotal = 0.0;
    for (NSDictionary *stat in iterations) {
        averageIORead += [[stat objectForKey:[PostingListModel STATISTIC_KEY_READ]] floatValue];
        averageIOReadWait += [[stat objectForKey:[PostingListModel STATISTIC_KEY_READ_WAIT]] floatValue];
        averagePreProcess += [[stat objectForKey:[PostingListModel STATISTIC_KEY_PREPROCESS]] floatValue];
        averageProcess += [[stat objectForKey:[PostingListModel STATISTIC_KEY_PROCESS]] floatValue];
        averagePostProcess += [[stat objectForKey:[PostingListModel STATISTIC_KEY_POSTPROCESS]] floatValue];
        averageProcessWait += [[stat objectForKey:[PostingListModel STATISTIC_KEY_PROCESS_WAIT]] floatValue];
        averageIOWrite += [[stat objectForKey:[PostingListModel STATISTIC_KEY_WRITE]] floatValue];
        averageIOWriteWait += [[stat objectForKey:[PostingListModel STATISTIC_KEY_WRITE_WAIT]] floatValue];
        averageTotal += [[stat objectForKey:[PostingListModel STATISTIC_KEY_TOTAL]] floatValue];
    }
    averageIORead /= _postingList.iterations;
    averageIOReadWait /= _postingList.iterations;
    averagePreProcess /= _postingList.iterations;
    averageProcess /= _postingList.iterations;
    averagePostProcess /= _postingList.iterations;
    averageProcessWait /= _postingList.iterations;
    averageIOWrite /= _postingList.iterations;
    averageIOWriteWait /= _postingList.iterations;
    averageTotal /= _postingList.iterations;
    
    NSMutableDictionary *stats = [NSMutableDictionary dictionary];
    [stats setValue:@(averageIORead) forKey:[PostingListModel STATISTIC_KEY_READ]];
    [stats setValue:@(averageIOReadWait) forKey:[PostingListModel STATISTIC_KEY_READ_WAIT]];
    [stats setValue:@(averagePreProcess) forKey:[PostingListModel STATISTIC_KEY_PREPROCESS]];
    [stats setValue:@(averageProcess) forKey:[PostingListModel STATISTIC_KEY_PROCESS]];
    [stats setValue:@(averagePostProcess) forKey:[PostingListModel STATISTIC_KEY_POSTPROCESS]];
    [stats setValue:@(averageProcessWait) forKey:[PostingListModel STATISTIC_KEY_PROCESS_WAIT]];
    [stats setValue:@(averageIOWrite) forKey:[PostingListModel STATISTIC_KEY_WRITE]];
    [stats setValue:@(averageIOWriteWait) forKey:[PostingListModel STATISTIC_KEY_WRITE_WAIT]];
    [stats setValue:@(averageTotal) forKey:[PostingListModel STATISTIC_KEY_TOTAL]];
    return stats;
}

@end
