// Copyright 2013 Atbrox AS
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#import "SecondViewController.h"

#import "PostingListModel.h"

@interface SecondViewController ()

- (void)setStatisticsView:(NSDictionary *)statistics;

@end

@implementation SecondViewController
{
    NSDictionary *_statistics;
    NSString *_operation;
    NSString *_scheme;
    NSUInteger _iterations;
    NSUInteger _blockSize;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
//    [self setStatisticsView:@"encode" statistics:_statistics];
//    [self setStatisticsView:@"decode" statistics:_statistics];
}

- (void)viewDidAppear:(BOOL)animated {
    [self setStatisticsView:_statistics];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)setDecodeStatistics:(NSDictionary *)statistics forScheme:(NSString *)scheme iterations:(NSUInteger)iterations blockSize:(NSUInteger)blockSize
{
    _iterations = iterations;
    _blockSize = blockSize;
    _statistics = statistics;
    _scheme = scheme;
    [self setStatisticsView:statistics];
    _operation = @"decode";
}



- (void)setEncodeStatistics:(NSDictionary *)statistics forScheme:(NSString *)scheme iterations:(NSUInteger)iterations blockSize:(NSUInteger)blockSize
{
    _iterations = iterations;
    _blockSize = blockSize;
    _statistics = statistics;
    _scheme = scheme;
    [self setStatisticsView:statistics];
    _operation = @"encode";
}

- (void)setStatisticsView:(NSDictionary *)statistics
{
    if (statistics)
    {
        NSNumber *readStat = [statistics objectForKey:PostingListModel.STATISTIC_KEY_READ];
        NSNumber *readWaitStat = [statistics objectForKey:PostingListModel.STATISTIC_KEY_READ_WAIT];
        NSNumber *preProcessStat = [statistics objectForKey:PostingListModel.STATISTIC_KEY_PREPROCESS];
        NSNumber *processStat = [statistics objectForKey:PostingListModel.STATISTIC_KEY_PROCESS];
        NSNumber *postProcessStat = [statistics objectForKey:PostingListModel.STATISTIC_KEY_POSTPROCESS];
        NSNumber *processWaitStat = [statistics objectForKey:PostingListModel.STATISTIC_KEY_PROCESS_WAIT];
        NSNumber *writeStat = [statistics objectForKey:PostingListModel.STATISTIC_KEY_WRITE];
        NSNumber *writeWaitStat = [statistics objectForKey:PostingListModel.STATISTIC_KEY_WRITE_WAIT];
        NSNumber *totalStat = [statistics objectForKey:PostingListModel.STATISTIC_KEY_TOTAL];
        
        //        NSLog(@"Read: %f. Pre-process: %f. Process: %f. Post-process: %f", readStat.stats.average, preProcessStat.stats.average, processStat.stats.average, postProcessStat.stats.average);
        
        [_ioReadTextField setText:[NSString stringWithFormat:@"%f", [readStat floatValue]]];
        [_ioReadWaitTextField setText:[NSString stringWithFormat:@"%f", [readWaitStat floatValue]]];
        
        [_preProcessTextField setText:[NSString stringWithFormat:@"%f", [preProcessStat floatValue]]];
        [_processTextField setText:[NSString stringWithFormat:@"%f", [processStat floatValue]]];
        [_postProcessTextField setText:[NSString stringWithFormat:@"%f", [postProcessStat floatValue]]];
        [_processWaitTextField setText:[NSString stringWithFormat:@"%f", [processWaitStat floatValue]]];
        
        [_ioWriteTextField setText:[NSString stringWithFormat:@"%f", [writeStat floatValue]]];
        [_ioWriteWaitTextField setText:[NSString stringWithFormat:@"%f", [writeWaitStat floatValue]]];
        
        [_totalTimeTextField setText:[NSString stringWithFormat:@"%f", [totalStat floatValue]]];
    }
}

- (IBAction) saveLog:(id)sender {
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    [formatter setDateFormat:@"yyyy-MM-dd_HH-mm-ss"];
    NSString *date = [formatter stringFromDate:[NSDate date]];
    NSUInteger suffix = 0;
    NSString *fileName = [NSString stringWithFormat:@"statistics-%@-%03i.log",
                          date,
                          suffix];
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *path = [[paths objectAtIndex:0] stringByAppendingPathComponent:fileName];
    NSFileManager *fm = [NSFileManager defaultManager];
    while ([fm fileExistsAtPath:path])
    {
        fileName = [NSString stringWithFormat:@"statistics-%@-%03i.log",
                    date,
                    ++suffix];
        path = [[paths objectAtIndex:0] stringByAppendingPathComponent:fileName];
        if (suffix > 999)
        {
            NSLog(@"Unable to save log file.");
            return;
        }
    }

    NSMutableString *logString = [NSMutableString stringWithFormat:@"Performed at: %@. Operation: %@. Scheme: %@. Block size: %i. Iterations: %i.\n", date, _operation, _scheme, _blockSize, _iterations];
    NSNumber *readStat = [_statistics objectForKey:PostingListModel.STATISTIC_KEY_READ];
    NSNumber *readWaitStat = [_statistics objectForKey:PostingListModel.STATISTIC_KEY_READ_WAIT];
    NSNumber *preProcessStat = [_statistics objectForKey:PostingListModel.STATISTIC_KEY_PREPROCESS];
    NSNumber *processStat = [_statistics objectForKey:PostingListModel.STATISTIC_KEY_PROCESS];
    NSNumber *postProcessStat = [_statistics objectForKey:PostingListModel.STATISTIC_KEY_POSTPROCESS];
    NSNumber *processWaitStat = [_statistics objectForKey:PostingListModel.STATISTIC_KEY_PROCESS_WAIT];
    NSNumber *writeStat = [_statistics objectForKey:PostingListModel.STATISTIC_KEY_WRITE];
    NSNumber *writeWaitStat = [_statistics objectForKey:PostingListModel.STATISTIC_KEY_WRITE_WAIT];
    NSNumber *totalStat = [_statistics objectForKey:PostingListModel.STATISTIC_KEY_TOTAL];
    
    [logString appendFormat:@"I/O Read: %@.\n"
                             "I/O Read Wait: %@.\n"
                             "Preprocessing: %@.\n"
                             "Processing: %@.\n"
                             "Postprocessing: %@.\n"
                             "Process Wait: %@.\n"
                             "I/O Write: %@.\n"
                             "I/O Write Wait: %@.\n"
                             "Total Execution Time: %@.\n",
     readStat, readWaitStat, preProcessStat, processStat, postProcessStat, processWaitStat, writeStat, writeWaitStat, totalStat
     ];

    NSLog(@"Storing log at '%@'...", path);
    [fm createFileAtPath:path contents:[logString dataUsingEncoding:NSUTF8StringEncoding] attributes:nil];
}

@end
