// Copyright 2013 Atbrox AS
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#import "CompressionToolbox.h"

@implementation CompressionToolbox

+ (NSUInteger)CLZ:(NSUInteger)number {
    NSUInteger leadingZeros;
#ifdef __arm__
//    NSLog(@"CLZ using ARM assembly...");
    asm("CLZ %0, %1" : "=r"(leadingZeros) : "r"(number));
#elif __i386__
//    NSLog(@"CLZ using x86 assembly...");
    NSUInteger log2;
    asm("BSR %0, %1" : "=r"(log2) : "r"(number));
    leadingZeros = 31 - log2;
#else
//    NSLog(@"CLZ using CLANG builtin...");
    leadingZeros = __builtin_clz(2);
#endif
    return leadingZeros;
}

+ (UInt8)reverseBits:(UInt8)byte {
    return BITREVERSE_TABLE[byte];
}

+ (NSData *)convertToBits:(NSData *)data overflow:(BOOL)overflow {
    
    NSUInteger bitLength = !overflow ? (data.length >> 3) : (data.length >> 3) + 1;
    BOOL *bits = (BOOL *)calloc(bitLength, sizeof(BOOL));
    BOOL *bytes = (BOOL *)data.bytes;
    
    for (NSInteger i = 0; i < data.length; i++) {
        BOOL bit = bytes[i];
        if (bit) {
            bits[i >> 3] |= 1 << (i & 7);
        }
        else {
            bits[i >> 3] &= ~(1 << (i & 7));
        }
    }
    return [NSData dataWithBytesNoCopy:bits length:bitLength freeWhenDone:YES];
}

+ (NSData *)convertToBytes:(NSData *)data from:(NSUInteger)from length:(NSUInteger)length {
    
    BOOL *bits = (BOOL *)data.bytes;
    BOOL *bytes = (BOOL *)calloc(length, sizeof(BOOL));
    
    for (NSUInteger i = from; i < length; i++) {
        BOOL bit = bits[i >> 3] & (1 << (i & 7));
        if (bit) {
            bytes[i] = YES;
        }
        else {
            bytes[i] = NO;
        }
    }
    
    return [NSData dataWithBytes:bytes length:length];
    
}

@end
