// Copyright 2013 Atbrox AS
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#import "EliasGammaEncoder.h"

#import "CompressionToolbox.h"

@interface EliasGammaEncoder ()

@end

@implementation EliasGammaEncoder

@synthesize encodedDataLength = _encodedDataLength;

- (NSUInteger)byteSize:(NSUInteger)number {
    NSUInteger bitLength = 31 - [CompressionToolbox CLZ:number];
    return (bitLength << 1) + 1;
}

- (NSData *)encodeNumber:(NSUInteger)number {
    const NSUInteger bitLength = 31 - [CompressionToolbox CLZ:number];
    const NSUInteger totalLength = (bitLength << 1) + 1;

    BOOL *bits = (BOOL *)calloc(totalLength, sizeof(BOOL));
    bits[bitLength] = 1;
    
    NSUInteger bitPosition = bitLength + 1;
    for (NSInteger i = bitLength - 1; i >= 0; i--) {
        if (number & 1 << i)
            bits[bitPosition] = 1;
        bitPosition++;
    }
    _encodedDataLength = bitPosition;
    return [NSData dataWithBytesNoCopy:bits length:totalLength freeWhenDone:YES];
}

- (NSData *)encode:(CFArrayRef)numbers {
    
    // Precalculate storage needed.
    
    NSUInteger totalLength = 0;
    NSUInteger itemCount = CFArrayGetCount(numbers);
    for (NSUInteger i = 0; i < itemCount; i++)
        totalLength += [self byteSize:(NSUInteger)CFArrayGetValueAtIndex(numbers, i)];
    
    NSUInteger currentPosition = 0;
    NSMutableData *data = [NSMutableData dataWithLength:totalLength];
    for (NSUInteger i = 0; i < itemCount; i++) {
            NSData *singleData = [self encodeNumber:(NSUInteger)CFArrayGetValueAtIndex(numbers, i)];
            [data replaceBytesInRange:NSMakeRange(currentPosition, singleData.length) withBytes:singleData.bytes];
            currentPosition += singleData.length;
    }
    _encodedDataLength = totalLength;
    return data;
}

- (NSString *)scheme {
    return @"Elias Gamma";
}

@end
