// Copyright 2013 Atbrox AS
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#import "GroupVarintDecoder.h"

@interface GroupVarintDecoder ()

- (NSUInteger)decodeBytes:(UInt8 *)buffer fromPosition:(NSUInteger)position length:(UInt8)length;

@end


@implementation GroupVarintDecoder

@synthesize decodedDataLength = _decodedDataLength;

- (NSUInteger)decodeBytes:(UInt8 *)buffer fromPosition:(NSUInteger)position length:(UInt8)length {

//    NSLog(@"Decoding at position %i with length %i", position, length);
    
    if (length == 1)
        return buffer[position];
    else if (length == 2)
        return buffer[position] | (buffer[position + 1] << 8);
    else if (length == 3)
        return buffer[position] | (buffer[position + 1] << 8) | (buffer[position + 2] << 16);
    else
        return buffer[position] | (buffer[position + 1] << 8) | (buffer[position + 2] << 16) | (buffer[position + 3] << 24);
}

- (CFMutableArrayRef)decode:(NSData *)data {
    
    CFMutableArrayRef numbers = CFArrayCreateMutable(kCFAllocatorDefault, 0, NULL);
    UInt8 *dataAsBytes = (UInt8 *)data.bytes;
    
    NSUInteger i = 0;
    
//    NSLog(@"Starting decode of data. Length: %i", data.length);
    
    while (i < data.length) {
        
        // Look up 1 byte mask and decode
        
        const UInt8 *keys = MASK_LOOKUP_TABLE[dataAsBytes[i]];
        i += 1;
        
        NSUInteger length = keys[0] + keys[1] + keys[2] + keys[3];
        if (i + length > data.length) {
            _decodedDataLength = i - 1;
            return numbers;
        }
        
//        NSNumber *a = [NSNumber numberWithUnsignedInteger:[self decodeBytes:dataAsBytes fromPosition:i length:keys[3]]];
        NSUInteger a = [self decodeBytes:dataAsBytes fromPosition:i length:keys[3]];
        CFArrayAppendValue(numbers, (void *)a);
        i += keys[3];
        
//        NSNumber *b = [NSNumber numberWithUnsignedInteger:[self decodeBytes:dataAsBytes fromPosition:i length:keys[2]]];
        NSUInteger b = [self decodeBytes:dataAsBytes fromPosition:i length:keys[2]];
        CFArrayAppendValue(numbers, (void *)b);
        i += keys[2];
        
//        NSNumber *c = [NSNumber numberWithUnsignedInteger:[self decodeBytes:dataAsBytes fromPosition:i length:keys[1]]];
        NSUInteger c = [self decodeBytes:dataAsBytes fromPosition:i length:keys[1]];
        CFArrayAppendValue(numbers, (void *)c);
        i += keys[1];
        
//        NSNumber *d = [NSNumber numberWithUnsignedInteger:[self decodeBytes:dataAsBytes fromPosition:i length:keys[0]]];
        NSUInteger d = [self decodeBytes:dataAsBytes fromPosition:i length:keys[0]];
        CFArrayAppendValue(numbers, (void *)d);
        i += keys[0];
        
        _decodedDataLength = i;
    }
    
//    NSLog(@"Exited with i = %i. Data length: %i. Length: %i", i, data.length, numbers.count);
    
    return numbers;
}

- (NSString *)scheme {
    return @"Group Varint";
}

@end
