// Copyright 2013 Atbrox AS
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#import <Foundation/Foundation.h>

#import "PostingListModelProtocol.h"
#import "RawPostingListReader.h"
#import "BundledRead.h"
#import "NSMutableArray+Queue.h"
#import "Statistics.h"
#import "Stat.h"
#import "PostingListEntry.h"

@class PostingListModel;

@protocol PostingListModelDelegate <NSObject>

- (void)readFinished:(PostingListModel *)model forPostingListEntry:(PostingListEntry *)entry;
- (void)preProcessingFinished:(PostingListModel *)model forPostingListEntry:(PostingListEntry *)entry;
- (void)processingFinished:(PostingListModel *)model forPostingListEntry:(PostingListEntry *)entry;
- (void)postProcessingFinished:(PostingListModel *)model forPostingListEntry:(PostingListEntry *)entry;
- (void)writeFinished:(PostingListModel *)model forPostingListEntry:(PostingListEntry *)entry;
                               
@end

@interface PostingListModel : NSObject <PostingListModelProtocol>

@property NSUInteger numberOfThreads;
@property (readonly) NSMutableDictionary *values, *stats;
@property (readonly) PostingListEntry *currentEntry;
@property (readonly) NSMutableArray *scheduledReads, *scheduledProcessing, *scheduledWrites;
@property NSString *path;
@property RawPostingListReader *reader;
@property NSFileHandle *writeDataFileHandle, *writeMappingFileHandle;
@property id <PostingListModelDelegate> delegate;

+ (NSString *)STATISTIC_KEY_READ;
+ (NSString *)STATISTIC_KEY_PREPROCESS;
+ (NSString *)STATISTIC_KEY_PROCESS;
+ (NSString *)STATISTIC_KEY_POSTPROCESS;
+ (NSString *)STATISTIC_KEY_WRITE;
+ (NSString *)STATISTIC_KEY_READ_WAIT;
+ (NSString *)STATISTIC_KEY_WRITE_WAIT;
+ (NSString *)STATISTIC_KEY_PROCESS_WAIT;
+ (NSString *)STATISTIC_KEY_TOTAL;

-(id) initWithPath:(NSString *)path;

-(NSUInteger) blockSize;
-(void) setBlockSize:(NSUInteger)blockSize;

-(void) readPostingListForTerms:(NSDictionary *)pairWiseTermAndByteRange;
-(NSUInteger) store:(NSString *)path withMappingAtPath:(NSString *)mappingPath inBlocks:(BOOL)blocked;

-(BOOL) _preprocess:(PostingListEntry *)preProcessObject;
-(BOOL) _process:(PostingListEntry *)processObject;
-(BOOL) _postprocess:(PostingListEntry *)postProcessObject;

@end
