// Copyright 2013 Atbrox AS
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#import <Foundation/Foundation.h>

#import "ParallelDecodingModel.h"
#import "ParallelEncodingModel.h"
#import "EncoderProtocol.h"
#import "DecoderProtocol.h"

@protocol PostingListCompressionDelegate <NSObject, PostingListModelDelegate>

- (void)iterationFinished:(NSUInteger)iteration operation:(NSString *)operation values:(NSDictionary *)values statistics:(NSDictionary *)stats;

@end

@interface PostingListCompression : NSObject

@property id <PostingListCompressionDelegate> delegate;

@property (readonly) ParallelEncodingModel *encodingModel;
@property (readonly) ParallelDecodingModel *decodingModel;
@property NSUInteger iterations;
@property NSUInteger blockSize;

+(NSRange) postingListRange:(NSString *)path forTerm:(NSString *)term atLineNo:(NSUInteger)lineNo withFixedByteSize:(NSUInteger)byteSize;
+(NSUInteger) numberOfTerms:(NSString *)path withFixedByteSize:(NSUInteger)byteSize;

-(void) setPathToUnencodedData:(NSString *)path;
-(NSString *)pathToUnencodedData;

-(void) setPathToEncodedData:(NSString *)path;
-(NSString *)pathToEncodedData;

-(void) addTermForEncode:(NSString *)term inRange:(NSRange)range;
-(void) addTermForDecode:(NSString *)term inRange:(NSRange)range;
-(void) encodeUsing:(id <EncoderProtocol>)encoder;
-(void) decodeUsing:(id <DecoderProtocol>)decoder;
-(void) storeDecodedData:(NSString *)path withMappingAtPath:(NSString *)mappingPath inBlocks:(BOOL)blocked;
-(void) storeEncodedData:(NSString *)path withMappingAtPath:(NSString *)mappingPath inBlocks:(BOOL)blocked;
-(NSDictionary *) decodeStatictics;
-(NSDictionary *) encodeStatistics;

@end
