// Copyright 2013 Atbrox AS
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#import "VariableByteEncoder.h"

@implementation VariableByteEncoder

@synthesize encodedDataLength = _encodedDataLength;

- (NSUInteger)byteSize:(NSUInteger)number
{
    int byteSize = 0;
    do {
        ++byteSize;
        number >>= 7;
    } while (number);
    return byteSize;
}

- (NSData *)encodeNumber:(NSUInteger)number
{
    // Determine number of bytes needed to store number
    NSInteger byteSize = [self byteSize:number];
    // NSLog(@"\nByte size of %li: %li", number, sz);
    
    NSMutableData *bytes = [NSMutableData dataWithLength:byteSize];
    NSInteger index = byteSize;
    while (YES) {
        Byte valueAsByte = number & 127;
        [bytes replaceBytesInRange:NSMakeRange(--index, 1) withBytes:&valueAsByte];
        // NSLog(@"Inserted value %i at position %li", value, index);
        if (number < 128)
            break;
        number >>= 7;
    }
    
    Byte lastByte;
    [bytes getBytes:&lastByte range:NSMakeRange(byteSize - 1, 1)];
    lastByte |= 128;
    [bytes replaceBytesInRange:NSMakeRange(byteSize - 1, 1) withBytes:&lastByte];
    return bytes;
}

- (NSData *)encode:(CFArrayRef)numbers {
    NSUInteger totalLength = 0;
    NSUInteger itemCount = CFArrayGetCount(numbers);
    for (NSUInteger i = 0; i < itemCount; i++) {
        totalLength += [self byteSize:(NSUInteger)CFArrayGetValueAtIndex(numbers, i)];
    }

    NSMutableData *byteStream = [NSMutableData dataWithLength:totalLength];
    NSUInteger currentPosition = 0;
    for (NSUInteger i = 0; i < itemCount; i++) {
        NSUInteger number = (NSUInteger)CFArrayGetValueAtIndex(numbers, i);
        NSData *bytes = [self encodeNumber:number];
        [byteStream replaceBytesInRange:NSMakeRange(currentPosition, bytes.length) withBytes:bytes.bytes];
        currentPosition += bytes.length;
    }
    
    _encodedDataLength = byteStream.length;
    
    return byteStream;
}

- (NSString *)scheme {
    return @"Variable Byte";
}

@end
