// Copyright 2013 Atbrox AS
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#import <Foundation/Foundation.h>

#import "ParallelEncodingModel.h"
#import "ParallelDecodingModel.h"
#import "PostingListEncoder.h"


NSString *postingListMappingPath = @"/Users/lars/Workspace/PostingList/PostingList/mapping-postinglist.out";
NSString *postingListPath = @"/Users/lars/Workspace/PostingList/PostingList/postinglist.out";
NSString *postingListEncodedPath = @"/Users/lars/Workspace/PostingList/PostingList/encoded-postinglist.out";
NSString *postingListEncodedMappingPath = @"/Users/lars/Workspace/PostingList/PostingList/mapping-encoded-postinglist.out";
NSString *postingListDecodedPath = @"/Users/lars/Workspace/PostingList/PostingList/decoded-postinglist.out";
NSString *postingListDecodedMappingPath = @"/Users/lars/Workspace/PostingList/PostingList/mapping-decoded-postinglist.out";


NSRange rangeForTerm(NSString *path, NSUInteger termLineNo, NSString *term)
{
    NSUInteger readLocation = (termLineNo - 1)*9;
    NSFileHandle *fileMappingHandle = [NSFileHandle fileHandleForReadingAtPath:path];
    [fileMappingHandle seekToFileOffset:readLocation];
    NSData *listStart = [fileMappingHandle readDataOfLength:8];
    [fileMappingHandle seekToFileOffset:readLocation + 9];
    NSData *listEnd = [fileMappingHandle readDataOfLength:8];
    NSUInteger start = [[[NSString alloc] initWithData:listStart encoding:NSASCIIStringEncoding] integerValue] + [term dataUsingEncoding:NSUTF8StringEncoding].length + 1;
    NSUInteger end = [[[NSString alloc] initWithData:listEnd encoding:NSASCIIStringEncoding] integerValue];
    
    return NSMakeRange(start, end - start - 1);
}


int main(int argc, const char * argv[])
{

    @autoreleasepool {
        
        ParallelEncodingModel *encodingModel = [[ParallelEncodingModel alloc] initWithPath:postingListPath];
        ParallelDecodingModel *decodingModel = [[ParallelDecodingModel alloc] initWithPath:postingListEncodedPath];
        [encodingModel setBlockSize:512];
        [decodingModel setBlockSize:512];
        NSString *term01 = @"00000001";
        NSString *term02 = @"00000002";
        NSDictionary *encodingTerms = [NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:
                                                                   [NSValue valueWithRange:rangeForTerm(postingListMappingPath, 1, term01)],
                                                                   [NSValue valueWithRange:rangeForTerm(postingListMappingPath, 2, term02)], nil]
                                                          forKeys:[NSArray arrayWithObjects:term01, term02, nil]];
        [encodingModel readPostingListForTerms:encodingTerms];
//        [model readPostingListForTerm:@"01" withinBytes:rangeForTerm(1) asListOfDelta:YES];
//        [model readPostingListForTerm:@"02" withinBytes:rangeForTerm(2) asListOfDelta:YES];
//        NSLog(@"%@", [model arrayForTerm:@"01"].debugDescription);
//        NSLog(@"%@", [model arrayForTerm:@"02"].debugDescription);
//        NSDictionary *encodedValues = [model getEncodedDataForAllTerms];
        sleep(5);
        [encodingModel store:postingListEncodedPath withMappingAtPath:postingListEncodedMappingPath inBlocks:YES];
        sleep(2);
        NSLog(@"\n\n### BEGIN DECODE ###\n\n");
        NSDictionary *decodingTerms = [NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:
                                                                           [NSValue valueWithRange:rangeForTerm(postingListEncodedMappingPath, 1, term01)],
                                                                           [NSValue valueWithRange:rangeForTerm(postingListEncodedMappingPath, 2, term02)], nil]
                                                                  forKeys:[NSArray arrayWithObjects:term01, term02, nil]];
        [decodingModel readPostingListForTerms:decodingTerms];
        sleep(5);
        [decodingModel store:postingListDecodedPath withMappingAtPath:postingListDecodedMappingPath inBlocks:YES];
        sleep(5);
    }
    return 0;
}

