// Copyright 2013 Atbrox AS
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#import "ViewController.h"
#import "FileReader.h"

@interface ViewController ()

@end

@implementation ViewController
{
    FileReader *reader;
    NSTimer *updateUITimer;
    BOOL random;
}

@synthesize gestureRecognizer, startButton, iterationsLabel, iterationsText, statusLabel, iterationProgress;
@synthesize totalProgress, totalProgressLabel, blockLabel, blockText, saveButton, randomLabel, randomToggle;

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    char *s = "";
    int i = strtol(s, NULL, 0);
    NSLog(@"i: %i", i);
    
    NSString *path = [[NSBundle mainBundle] pathForResource:@"data" ofType:@"random"];
    reader = [[FileReader alloc] initFromPath:path];
    iterationsText.text = @"100";
    blockText.text = @"1";
    reader.iterations = 100;
    reader.blockSize = 1;
    random = NO;

}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

-(BOOL) textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string
{
    NSMutableString *text = [NSMutableString stringWithString:textField.text];
    if (![string compare:@""])
        [text deleteCharactersInRange:NSMakeRange(text.length - 1, 1)];
    else
        [text appendString:string];
    NSInteger value = [text integerValue];
    if (textField.tag == 0)
    {
        if (value < 0)
            reader.iterations = 0;
        else
            reader.iterations = value;
    }
    else if (textField.tag == 1)
    {
        if (value <= 0)
            reader.blockSize = 1;
        else
            reader.blockSize = value;
    }
    return YES;
}

-(IBAction) start:(id)sender
{
    // Disable controls
    iterationsText.userInteractionEnabled = NO;
    blockText.userInteractionEnabled = NO;
    startButton.userInteractionEnabled = NO;
    saveButton.userInteractionEnabled = NO;
    // Show status
    statusLabel.hidden = NO;
    iterationProgress.hidden = NO;
    totalProgress.hidden = NO;
    totalProgressLabel.hidden = NO;
    // Set values
    statusLabel.text = @"";
    iterationProgress.progress = 0.0;
    totalProgress.progress = 0.0;
    [startButton setTitle:@"Running..." forState:UIControlStateNormal];
    
    // Start reading file
    dispatch_queue_t readQueue = dispatch_queue_create("ReadQueue", NULL);
    dispatch_async(readQueue, ^{
        [reader read:random];
    });
    
//    [reader resetCounters];
    updateUITimer = [NSTimer scheduledTimerWithTimeInterval:0.02 target:self selector:@selector(updateProgressUI) userInfo:nil repeats:YES];
}

-(void)updateProgressUI
{
    if (updateUITimer)
    {
        if (reader.currentIteration < reader.iterations)
        {
            //NSLog(@"Setting progress: Current read bytes: %i", reader.currentReadBytes);
            float floatIterationProgress = reader.currentReadBytes / (float)reader.totalBytes;
            float floatTotalProgress = (reader.currentIteration / (float)reader.iterations) + (floatIterationProgress / reader.iterations);
            [iterationProgress setProgress:floatIterationProgress animated:YES];
            [totalProgress setProgress:floatTotalProgress animated:YES];
            statusLabel.text = [NSString stringWithFormat:@"%i / %i\nRead %i / %i bytes", reader.currentIteration + 1, reader.iterations, reader.currentReadBytes, reader.totalBytes];
        }
        else
        {
            [updateUITimer invalidate];
            [totalProgress setProgress:reader.currentIteration / (float)reader.iterations animated:YES];
            iterationsText.userInteractionEnabled = YES;
            blockText.userInteractionEnabled = YES;
            startButton.userInteractionEnabled = YES;
            saveButton.userInteractionEnabled = YES;
            [startButton setTitle:@"Start" forState:UIControlStateNormal];
            
        }
    }
}

-(IBAction) toggleRandom:(id)sender
{
    random = [sender isOn];
}

-(IBAction) dismissKeyboard:(id)sender
{
    [iterationsText resignFirstResponder];
    [blockText resignFirstResponder];
}

-(IBAction) saveLog:(id)sender
{
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    [formatter setDateFormat:@"yyyy-MM-dd_HH-mm-ss"];
    NSUInteger suffix = 0;
    NSString *fileName = [NSString stringWithFormat:@"%@-%i-%i-%03i.log", [formatter stringFromDate:reader.start], reader.iterations, reader.blockSize, suffix];
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *path = [[paths objectAtIndex:0] stringByAppendingPathComponent:fileName];
    NSFileManager *fm = [NSFileManager defaultManager];
    while ([fm fileExistsAtPath:path])
    {
        fileName = [NSString stringWithFormat:@"%@-%i-%i-%03i.log", [formatter stringFromDate:reader.start], reader.iterations, reader.blockSize, ++suffix];
        path = [[paths objectAtIndex:0] stringByAppendingPathComponent:fileName];
        if (suffix > 999)
        {
            NSLog(@"Unable to save log file.");
            return;
        }
    }
    [reader writeLog:path];
}

@end
